@@ -23,17 +23,12 @@
  * We also do some non-VM-related chores, such as releasing the cred pointer
  * (for AIX and Solaris) and releasing the gnode (for AIX).
  *
- * Locking:  afs_xvcache lock is held.  If it is dropped and re-acquired,
- *   *slept should be set to warn the caller.
- *
- * Formerly, afs_xvcache was dropped and re-acquired for Solaris, but now it
- * is not dropped and re-acquired for any platform.  It may be that *slept is
- * therefore obsolescent.
+ * Locking:  afs_xvcache lock is held. It must not be dropped.
  *
  * OSF/1 Locking:  VN_LOCK has been called.
  */
 int
-osi_VM_FlushVCache(struct vcache *avc, int *slept)
+osi_VM_FlushVCache(struct vcache *avc)
 {
     struct vnode *vp = AFSTOV(avc);
     kern_return_t kret;
@@ -42,12 +37,6 @@ osi_VM_FlushVCache(struct vcache *avc, int *slept)
     if (!vp)
 	return 0;
     AFS_GUNLOCK();
-#if 0
-    if (!(UBCINFOMISSING(vp) || UBCINFORECLAIMED(vp))) {
-      size=ubc_getsize(vp);
-      kret=ubc_invalidate(vp,0,size); 
-    }
-#endif
     cache_purge(vp);
     AFS_GLOCK();
 
@@ -87,7 +76,7 @@ osi_VM_StoreAllSegments(struct vcache *avc)
  * be some pages around when we return, newly created by concurrent activity.
  */
 void
-osi_VM_TryToSmush(struct vcache *avc, struct AFS_UCRED *acred, int sync)
+osi_VM_TryToSmush(struct vcache *avc, afs_ucred_t *acred, int sync)
 {
     struct vnode *vp = AFSTOV(avc);
     void *object;
@@ -118,7 +107,7 @@ osi_VM_TryToSmush(struct vcache *avc, struct AFS_UCRED *acred, int sync)
    this, and also making sure that ubc's idea of the filesize is right more
    often */
 void
-osi_VM_FlushPages(struct vcache *avc, struct AFS_UCRED *credp)
+osi_VM_FlushPages(struct vcache *avc, afs_ucred_t *credp)
 {
     struct vnode *vp = AFSTOV(avc);
     void *object;
@@ -151,7 +140,7 @@ osi_VM_FlushPages(struct vcache *avc, struct AFS_UCRED *credp)
  * it only works on Solaris.
  */
 void
-osi_VM_Truncate(struct vcache *avc, int alen, struct AFS_UCRED *acred)
+osi_VM_Truncate(struct vcache *avc, int alen, afs_ucred_t *acred)
 {
     struct vnode *vp = AFSTOV(avc);
 #ifdef AFS_DARWIN80_ENV
@@ -166,19 +155,6 @@ osi_VM_Truncate(struct vcache *avc, int alen, struct AFS_UCRED *acred)
 void
 osi_VM_NukePages(struct vnode *vp, off_t offset, off_t size)
 {
-#if 0
-    void *object;
-    struct vcache *avc = VTOAFS(vp);
-
-    offset = trunc_page(offset);
-    size = round_page(size + 1);
-    while (size) {
-	ubc_page_op(vp, (vm_offset_t) offset,
-		    UPL_POP_SET | UPL_POP_BUSY | UPL_POP_DUMP, 0, 0);
-	size -= PAGE_SIZE;
-	offset += PAGE_SIZE;
-    }
-#endif
 }
 
 int
