@@ -10,7 +10,7 @@
 
 /* osi_vm.c implements:
  *
- * osi_VM_FlushVCache(avc, slept)
+ * osi_VM_FlushVCache(avc)
  * osi_ubc_flush_dirty_and_wait(vp, flags)
  * osi_VM_StoreAllSegments(avc)
  * osi_VM_TryToSmush(avc, acred, sync)
@@ -29,12 +29,15 @@
 #include "afs/afs_stats.h"	/* statistics */
 #include <vm/vm_object.h>
 #include <vm/vm_map.h>
-#include <limits.h>
+#include <sys/limits.h>
+#if __FreeBSD_version >= 1000030
+#include <sys/rwlock.h>
+#endif
 
 /*
  * FreeBSD implementation notes:
  * Most of these operations require us to frob vm_objects.  Most
- * functions require that the object be locked (with VM_OBJECT_LOCK)
+ * functions require that the object be locked (with VM_OBJECT_*LOCK)
  * on entry and leave it locked on exit.  The locking protocol
  * requires that we access vp->v_object with the heavy vnode lock
  * held and the vnode interlock unlocked.
@@ -45,16 +48,17 @@
  * check the VCS history of those files.
  */
 
-#if defined(AFS_FBSD80_ENV)
 #define	lock_vnode(v, f)	vn_lock((v), (f))
-#define ilock_vnode(v)	vn_lock((v), LK_INTERLOCK|LK_EXCLUSIVE|LK_RETRY);
+#define ilock_vnode(v)	vn_lock((v), LK_INTERLOCK|LK_EXCLUSIVE|LK_RETRY)
 #define unlock_vnode(v)	VOP_UNLOCK((v), 0)
 #define islocked_vnode(v)	VOP_ISLOCKED((v))
+
+#if __FreeBSD_version >= 1000030
+#define AFS_VM_OBJECT_WLOCK(o)	VM_OBJECT_WLOCK(o)
+#define AFS_VM_OBJECT_WUNLOCK(o)	VM_OBJECT_WUNLOCK(o)
 #else
-#define	lock_vnode(v, f)	vn_lock((v), (f), curthread)
-#define ilock_vnode(v)	vn_lock((v), LK_INTERLOCK|LK_EXCLUSIVE|LK_RETRY, curthread);
-#define unlock_vnode(v)	VOP_UNLOCK((v), 0, curthread)
-#define islocked_vnode(v)	VOP_ISLOCKED((v), curthread)
+#define AFS_VM_OBJECT_WLOCK(o)	VM_OBJECT_LOCK(o)
+#define AFS_VM_OBJECT_WUNLOCK(o)	VM_OBJECT_UNLOCK(o)
 #endif
 
 /* Try to discard pages, in order to recycle a vcache entry.
@@ -64,59 +68,36 @@
  * We also do some non-VM-related chores, such as releasing the cred pointer
  * (for AIX and Solaris) and releasing the gnode (for AIX).
  *
- * Locking:  afs_xvcache lock is held.  If it is dropped and re-acquired,
- *   *slept should be set to warn the caller.
- *
- * Formerly, afs_xvcache was dropped and re-acquired for Solaris, but now it
- * is not dropped and re-acquired for any platform.  It may be that *slept is
- * therefore obsolescent.
+ * Locking:  afs_xvcache lock is held. It must not be dropped.
  *
  */
 int
-osi_VM_FlushVCache(struct vcache *avc, int *slept)
+osi_VM_FlushVCache(struct vcache *avc)
 {
-    struct vnode *vp = AFSTOV(avc);
+    struct vnode *vp;
+    int code;
 
-    if (!VI_TRYLOCK(vp)) /* need interlock to check usecount */
+    vp = AFSTOV(avc);
+
+    if (!VI_TRYLOCK(vp))
 	return EBUSY;
-
-    if (vp->v_usecount > 0) {
+    code = osi_fbsd_checkinuse(avc);
+    if (code) {
 	VI_UNLOCK(vp);
-	return EBUSY;
+	return code;
     }
 
-    /* XXX
-     * The value of avc->opens here came to be, at some point,
-     * typically -1.  This was caused by incorrectly performing afs_close
-     * processing on vnodes being recycled */
-    if (avc->opens) {
-	VI_UNLOCK(vp);
-	return EBUSY;
-    }
-
-    /* if a lock is held, give up */
-    if (CheckLock(&avc->lock)) {
-	VI_UNLOCK(vp);
-	return EBUSY;
-    }
-
-    if ((vp->v_iflag & VI_DOOMED) != 0) {
-	VI_UNLOCK(vp);
-	return (0);
-    }
-
-    /* must hold the vnode before calling vgone()
+    /* must hold the vnode before calling cache_purge()
      * This code largely copied from vfs_subr.c:vlrureclaim() */
     vholdl(vp);
+    VI_UNLOCK(vp);
+
     AFS_GUNLOCK();
-    *slept = 1;
-    /* use the interlock while locking, so no one else can DOOM this */
-    ilock_vnode(vp);
-    vgone(vp);
-    unlock_vnode(vp);
+    cache_purge(vp);
+    AFS_GLOCK();
+
     vdrop(vp);
 
-    AFS_GLOCK();
     return 0;
 }
 
@@ -154,9 +135,9 @@ osi_VM_StoreAllSegments(struct vcache *avc)
 	    if (!vget(vp, LK_EXCLUSIVE | LK_RETRY, curthread)) {
 		    obj = vp->v_object;
 		    if (obj != NULL) {
-			VM_OBJECT_LOCK(obj);
+			AFS_VM_OBJECT_WLOCK(obj);
 			vm_object_page_clean(obj, 0, 0, OBJPC_SYNC);
-			VM_OBJECT_UNLOCK(obj);
+			AFS_VM_OBJECT_WUNLOCK(obj);
 			anyio = 1;
 		    }
 		    vput(vp);
@@ -202,7 +183,7 @@ osi_VM_TryToSmush(struct vcache *avc, afs_ucred_t *acred, int sync)
 	lock_vnode(vp, LK_EXCLUSIVE);
 
     if (vp->v_bufobj.bo_object != NULL) {
-	VM_OBJECT_LOCK(vp->v_bufobj.bo_object);
+	AFS_VM_OBJECT_WLOCK(vp->v_bufobj.bo_object);
 	/*
 	 * Do we really want OBJPC_SYNC?  OBJPC_INVAL would be
 	 * faster, if invalidation is really what we are being
@@ -218,7 +199,7 @@ osi_VM_TryToSmush(struct vcache *avc, afs_ucred_t *acred, int sync)
 	 */
 
 	vm_object_page_clean(vp->v_bufobj.bo_object, 0, 0, OBJPC_SYNC);
-	VM_OBJECT_UNLOCK(vp->v_bufobj.bo_object);
+	AFS_VM_OBJECT_WUNLOCK(vp->v_bufobj.bo_object);
     }
 
     tries = 5;
@@ -248,9 +229,9 @@ osi_VM_FlushPages(struct vcache *avc, afs_ucred_t *credp)
     ASSERT_VOP_LOCKED(vp, __func__);
     obj = vp->v_object;
     if (obj != NULL) {
-	VM_OBJECT_LOCK(obj);
+	AFS_VM_OBJECT_WLOCK(obj);
 	vm_object_page_remove(obj, 0, 0, FALSE);
-	VM_OBJECT_UNLOCK(obj);
+	AFS_VM_OBJECT_WUNLOCK(obj);
     }
     osi_vinvalbuf(vp, 0, 0, 0);
 }
