@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -38,7 +38,7 @@
 
 #define AFS_KALLOC(A)           afs_osi_Alloc(A)
 
-/* 
+/*
  * Time related macros
  */
 #define	afs_hz	    HZ
@@ -51,9 +51,6 @@
 
 #define osi_vnhold(avc, r)  do { VN_HOLD(AFSTOV(avc)); } while(0)
 
-#if 0
-#define	afs_suser(x)	    suser(x)
-#endif
 /*
  * Global lock support.
  */
@@ -83,4 +80,9 @@ extern int afs_bufferpages;
 
 extern int afs_suser(void *credp);
 
+#define setuerror(erval) get_user_struct()->u_error = (erval)
+#define getuerror(erval) get_user_struct()->u_error
+#define osi_curcred()    get_user_struct()->u_cred
+
+#define osi_procname(procname, size) strncpy(procname, "(unknown)", size)
 #endif /* _OSI_MACHDEP_H_ */
