@@ -33,6 +33,7 @@
 #include "h/ksynch.h"
 #include "h/sunddi.h"
 #endif
+#include <hcrypto/rand.h>
 
 #if defined(AFS_SUN5_ENV) || defined(AFS_AIX_ENV) || defined(AFS_SGI_ENV) || defined(AFS_HPUX_ENV)
 #define	AFS_MINBUFFERS	100
@@ -40,7 +41,7 @@
 #define	AFS_MINBUFFERS	50
 #endif
 
-#if (defined(AFS_SUN5_ENV) || (defined(AFS_LINUX24_ENV) && defined(HAVE_LINUX_COMPLETION_H)) || defined(AFS_DARWIN80_ENV)) && !defined(UKERNEL)
+#if (defined(AFS_SUN5_ENV) || defined(AFS_LINUX26_ENV) || defined(AFS_DARWIN80_ENV)) && !defined(UKERNEL)
 /* If AFS_DAEMONOP_ENV is defined, it indicates we run "daemon" AFS syscalls by
  * spawning a kernel thread to do the work, instead of running them in the
  * calling process. */
@@ -63,7 +64,6 @@ krwlock_t afsifinfo_lock;
 
 afs_int32 afs_initState = 0;
 afs_int32 afs_termState = 0;
-afs_int32 afs_setTime = 0;
 int afs_cold_shutdown = 0;
 char afs_SynchronousCloses = '\0';
 static int afs_CB_Running = 0;
@@ -87,6 +87,8 @@ afs_int32 afs_rx_idledead_rep = AFS_IDLEDEADTIME_REP;
 
 static int afscall_set_rxpck_received = 0;
 
+extern afs_int32 afs_volume_ttl;
+
 /* From afs_util.c */
 extern afs_int32 afs_md5inum;
 
@@ -122,8 +124,6 @@ afs_InitSetup(int preallocs)
 
     memset(afs_zeros, 0, AFS_ZEROS);
 
-    rx_SetBusyChannelError(1);  /* turn on busy call error reporting */
-
     /* start RX */
     if(!afscall_set_rxpck_received)
     rx_extraPackets = AFS_NRXPACKETS;	/* smaller # of packets */
@@ -281,9 +281,9 @@ afs_DaemonOp(long parm, long parm2, long parm3, long parm4, long parm5,
 #endif
 
 
-#if defined(AFS_LINUX24_ENV) && defined(HAVE_LINUX_COMPLETION_H)
+#if defined(AFS_LINUX26_ENV)
 struct afsd_thread_info {
-# if defined(AFS_LINUX26_ENV) && !defined(INIT_WORK_HAS_DATA)
+# if !defined(INIT_WORK_HAS_DATA)
     struct work_struct tq;
 # endif
     unsigned long parm;
@@ -345,6 +345,9 @@ afsd_thread(void *rock)
 	complete_and_exit(0, 0);
 	break;
     case AFSOP_START_BKG:
+#ifdef AFS_NEW_BKG
+	afs_warn("Install matching afsd! Old background daemons not supported.\n");
+#else
 	sprintf(current->comm, "afs_bkgstart");
 	AFS_GLOCK();
 	complete(arg->complete);
@@ -357,6 +360,7 @@ afsd_thread(void *rock)
 	sprintf(current->comm, "afs_background");
 	afs_BackgroundDaemon();
 	AFS_GUNLOCK();
+#endif
 	complete_and_exit(0, 0);
 	break;
     case AFSOP_START_TRUNCDAEMON:
@@ -596,6 +600,268 @@ afs_DaemonOp(long parm, long parm2, long parm3, long parm4, long parm5,
 }
 #endif /* AFS_SUN5_ENV */
 
+#ifdef AFS_SOCKPROXY_ENV
+/**
+ * Deallocate packets.
+ *
+ * @param[inout]  a_pktlist  list of packets to be freed
+ * @param[in]     a_npkts    number of packets
+ */
+static void
+sockproxy_pkts_free(struct afs_pkt_hdr **a_pktlist, size_t a_npkts)
+{
+    int pkt_i;
+    struct afs_pkt_hdr *pktlist = *a_pktlist;
+
+    if (pktlist == NULL) {
+	return;
+    }
+    *a_pktlist = NULL;
+
+    for (pkt_i = 0; pkt_i < a_npkts; pkt_i++) {
+	struct afs_pkt_hdr *pkt = &pktlist[pkt_i];
+	afs_osi_Free(pkt->payload, pkt->size);
+	pkt->payload = NULL;
+    }
+    afs_osi_Free(pktlist, a_npkts * sizeof(pktlist[0]));
+}
+
+/**
+ * Copy packets from user-space.
+ *
+ * @param[in]   a_uaddr    address of list of packets in user-space
+ * @param[out]  a_pktlist  packets copied from user-space
+ * @param[in]   a_npkts    number of packets to be copied
+ *
+ * @return errno error codes.
+ */
+static int
+sockproxy_pkts_copyin(user_addr_t a_uaddr, struct afs_pkt_hdr **a_pktlist,
+		      size_t a_npkts)
+{
+    int code, pkt_i;
+    struct afs_pkt_hdr *pktlist_u;
+    struct afs_pkt_hdr *pktlist_k = NULL;
+
+    *a_pktlist = NULL;
+    if (a_npkts == 0) {
+	return 0;
+    }
+    if (a_npkts > AFS_SOCKPROXY_PKT_MAX) {
+	return E2BIG;
+    }
+
+    pktlist_u = afs_osi_Alloc(a_npkts * sizeof(pktlist_u[0]));
+    if (pktlist_u == NULL) {
+	code = ENOMEM;
+	goto done;
+    }
+
+    pktlist_k = afs_osi_Alloc(a_npkts * sizeof(pktlist_k[0]));
+    if (pktlist_k == NULL) {
+	code = ENOMEM;
+	goto done;
+    }
+    memset(pktlist_k, 0, a_npkts * sizeof(pktlist_k[0]));
+
+    AFS_COPYIN(a_uaddr, pktlist_u, a_npkts * sizeof(pktlist_u[0]), code);
+    if (code != 0) {
+	goto done;
+    }
+
+    for (pkt_i = 0; pkt_i < a_npkts; pkt_i++) {
+	struct afs_pkt_hdr *pkt_k = &pktlist_k[pkt_i];
+	struct afs_pkt_hdr *pkt_u = &pktlist_u[pkt_i];
+
+	if (pkt_u->size > AFS_SOCKPROXY_PAYLOAD_MAX) {
+	    code = E2BIG;
+	    goto done;
+	}
+	/*
+	 * Notice that pkt_u->payload points to a user-space address. When
+	 * copying the data from pkt_u to pkt_k, make sure that pkt_k->payload
+	 * points to a valid address in kernel-space.
+	 */
+	*pkt_k = *pkt_u;
+	pkt_k->payload = afs_osi_Alloc(pkt_k->size);
+	if (pkt_k->payload == NULL) {
+	    code = ENOMEM;
+	    goto done;
+	}
+
+	AFS_COPYIN((user_addr_t)pkt_u->payload, pkt_k->payload, pkt_k->size,
+		   code);
+	if (code != 0) {
+	    goto done;
+	}
+    }
+
+    *a_pktlist = pktlist_k;
+    pktlist_k = NULL;
+    code = 0;
+
+ done:
+    sockproxy_pkts_free(&pktlist_k, a_npkts);
+    afs_osi_Free(pktlist_u, a_npkts * sizeof(pktlist_u[0]));
+
+    return code;
+}
+
+/**
+ * Copy packets to user-space.
+ *
+ * @param[in]  a_uaddr    dst address of list of packets in user-space
+ * @param[in]  a_pktlist  packets to be copied to user-space
+ * @param[in]  a_npkts    number of packets to be copied
+ *
+ * @return 0 on success; non-zero otherwise.
+ */
+static int
+sockproxy_pkts_copyout(user_addr_t a_uaddr, struct afs_pkt_hdr *a_pktlist,
+		       size_t a_npkts)
+{
+    int code, pkt_i;
+    struct afs_pkt_hdr *pktlist_u = NULL;
+    struct afs_pkt_hdr *pktlist_k = a_pktlist;
+
+    if (a_npkts == 0) {
+	return 0;
+    }
+    if (a_npkts > AFS_SOCKPROXY_PKT_MAX) {
+	return E2BIG;
+    }
+    if (a_pktlist == NULL) {
+	return EINVAL;
+    }
+
+    pktlist_u = afs_osi_Alloc(a_npkts * sizeof(pktlist_u[0]));
+    if (pktlist_u == NULL) {
+	code = ENOMEM;
+	goto done;
+    }
+
+    AFS_COPYIN(a_uaddr, pktlist_u, a_npkts * sizeof(pktlist_u[0]), code);
+    if (code != 0) {
+	goto done;
+    }
+
+    for (pkt_i = 0; pkt_i < a_npkts; pkt_i++) {
+	struct afs_pkt_hdr *pkt_k = &pktlist_k[pkt_i];
+	struct afs_pkt_hdr *pkt_u = &pktlist_u[pkt_i];
+	void *payload_uaddr;
+
+	if (pkt_k->size > pkt_u->size) {
+	    code = ENOSPC;
+	    goto done;
+	}
+
+	/* Copy pkt_k -> pkt_u, but preserve pkt_u->payload */
+	payload_uaddr = pkt_u->payload;
+	*pkt_u = *pkt_k;
+	pkt_u->payload = payload_uaddr;
+
+	AFS_COPYOUT(pkt_k->payload, (user_addr_t)pkt_u->payload, pkt_k->size,
+		    code);
+	if (code != 0) {
+	    goto done;
+	}
+    }
+
+    AFS_COPYOUT(pktlist_u, a_uaddr, a_npkts * sizeof(pktlist_u[0]), code);
+    if (code != 0) {
+	goto done;
+    }
+
+ done:
+    afs_osi_Free(pktlist_u, a_npkts * sizeof(pktlist_u[0]));
+    return code;
+}
+
+/**
+ * Receive / send packets from / to user-space.
+ *
+ * @param[in]  a_parm_uspc  afs_uspc_param struct
+ * @param[in]  a_parm_pkts  packets to be received / sent
+ *
+ * @return 0 on success; non-zero otherwise.
+ */
+static int
+sockproxy_handler(user_addr_t a_parm_uspc, user_addr_t a_parm_pkts)
+{
+    int code;
+    size_t orig_npkts, npkts;
+    struct afs_uspc_param uspc;
+
+    struct afs_pkt_hdr *pkts_recv;
+    struct afs_pkt_hdr *pkts_send;
+
+    AFS_GUNLOCK();
+
+    orig_npkts = 0;
+    memset(&uspc, 0, sizeof(uspc));
+
+    pkts_recv = NULL;
+    pkts_send = NULL;
+
+    /* get response from user-space */
+    AFS_COPYIN(a_parm_uspc, &uspc, sizeof(uspc), code);
+    if (code != 0) {
+	afs_warn("afs: AFSOP_SOCKPROXY_HANDLER can't read uspc\n");
+	goto done;
+    }
+
+    npkts = uspc.req.usp.npkts;
+    orig_npkts = npkts;
+
+    if (uspc.reqtype == AFS_USPC_SOCKPROXY_RECV && npkts > 0) {
+	/* copyin packets in from user-space */
+	code = sockproxy_pkts_copyin(a_parm_pkts, &pkts_recv, npkts);
+	if (code) {
+	    afs_warn("afs: AFSOP_SOCKPROXY_HANDLER can't read pkts\n");
+	    goto done;
+	}
+    }
+
+    /*
+     * send response from user-space (if any) to the rx layer and wait for a
+     * new request.
+     */
+    code = rxk_SockProxyReply(&uspc, pkts_recv, &pkts_send);
+    if (code) {
+	afs_warn("afs: AFSOP_SOCKPROXY_HANDLER rxk_SockProxyReply failed\n");
+	goto done;
+    }
+
+    /* send request to user-space process */
+    AFS_COPYOUT(&uspc, a_parm_uspc, sizeof(uspc), code);
+    if (code) {
+	afs_warn("afs: AFSOP_SOCKPROXY_HANDLER can't write uspc\n");
+	goto done;
+    }
+
+    npkts = uspc.req.usp.npkts;
+    if (uspc.reqtype == AFS_USPC_SOCKPROXY_SEND && npkts > 0) {
+	/* check if process allocated enough memory to receive the packets */
+	if (npkts > orig_npkts) {
+	    code = ENOSPC;
+	    goto done;
+	}
+
+	/* copyout packets to user-space */
+	code = sockproxy_pkts_copyout(a_parm_pkts, pkts_send, npkts);
+	if (code != 0) {
+	    afs_warn("afs: AFSOP_SOCKPROXY_HANDLER can't write pkts\n");
+	    goto done;
+	}
+    }
+ done:
+    sockproxy_pkts_free(&pkts_recv, orig_npkts);
+    AFS_GLOCK();
+
+    return code;
+}
+#endif /* AFS_SOCKPROXY_ENV */
+
 #ifdef AFS_DARWIN100_ENV
 # define AFSKPTR(X) k ## X
 int
@@ -622,7 +888,7 @@ afs_syscall_call(long parm, long parm2, long parm3,
 #endif
 {
     afs_int32 code = 0;
-#if defined(AFS_SGI61_ENV) || defined(AFS_SUN57_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
+#if defined(AFS_SGI61_ENV) || defined(AFS_SUN5_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
     size_t bufferSize;
 #else /* AFS_SGI61_ENV */
     u_int bufferSize;
@@ -665,7 +931,7 @@ afs_syscall_call(long parm, long parm2, long parm3,
     put_vfs_context();
 #endif
 #ifdef AFS_DAEMONOP_ENV
-# if defined(AFS_DARWIN80_ENV)
+# if defined(AFS_NEW_BKG)
     if (parm == AFSOP_BKG_HANDLER) {
 	/* if afs_uspc_param grows this should be checked */
 	struct afs_uspc_param *mvParam = osi_AllocSmallSpace(AFS_SMALLOCSIZ);
@@ -708,7 +974,7 @@ afs_syscall_call(long parm, long parm2, long parm3,
 	afs_osi_Free(param2, namebufsz);
 	osi_FreeSmallSpace(mvParam);
     } else
-# endif /* DARWIN80 */
+# endif /* AFS_NEW_BKG */
     if (parm < AFSOP_ADDCELL || parm == AFSOP_RXEVENT_DAEMON
 	|| parm == AFSOP_RXLISTENER_DAEMON) {
 	afs_DaemonOp(parm, parm2, parm3, parm4, parm5, parm6);
@@ -800,7 +1066,7 @@ afs_syscall_call(long parm, long parm2, long parm3,
 	AFS_GUNLOCK();
 	exit(CLD_EXITED, 0);
 # endif /* AFS_SGI_ENV */
-# ifndef AFS_DARWIN80_ENV
+# ifndef AFS_NEW_BKG
     } else if (parm == AFSOP_START_BKG) {
 	while (afs_initState < AFSOP_START_BKG)
 	    afs_osi_Sleep(&afs_initState);
@@ -821,7 +1087,7 @@ afs_syscall_call(long parm, long parm2, long parm3,
 	AFS_GUNLOCK();
 	exit(CLD_EXITED, 0);
 #  endif /* AFS_SGI_ENV */
-# endif /* ! AFS_DARWIN80_ENV */
+# endif /* ! AFS_NEW_BKG */
     } else if (parm == AFSOP_START_TRUNCDAEMON) {
 	while (afs_initState < AFSOP_GO)
 	    afs_osi_Sleep(&afs_initState);
@@ -991,7 +1257,10 @@ afs_syscall_call(long parm, long parm2, long parm3,
 	}
 	afs_CacheInit_Done = 1;
         code = afs_icl_InitLogs();
-	afs_setTime = cparms.setTimeFlag;
+	if (cparms.setTimeFlag) {
+	    afs_warn("afs: AFSOP_CACHEINIT setTimeFlag ignored; are you "
+	             "running an old afsd?\n");
+	}
 
 	code =
 	    afs_CacheInit(cparms.cacheScaches, cparms.cacheFiles,
@@ -1070,7 +1339,11 @@ afs_syscall_call(long parm, long parm2, long parm3,
 	while (afs_initState < AFSOP_GO)
 	    afs_osi_Sleep(&afs_initState);
 	afs_initState = 101;
-	afs_setTime = parm2;
+	if (parm2) {
+	    /* parm2 used to set afs_setTime */
+	    afs_warn("afs: AFSOP_GO setTime flag ignored; are you running an "
+	             "old afsd?\n");
+	}
 	if (afs_tpct1 + afs_tpct2 != 100) {
 	    afs_tpct1 = 0;
 	    afs_tpct2 = 0;
@@ -1085,6 +1358,10 @@ afs_syscall_call(long parm, long parm2, long parm3,
 	if (afs_uuid_create(&afs_cb_interface.uuid) != 0)
 	    memset(&afs_cb_interface.uuid, 0, sizeof(afsUUID));
 
+#if defined(AFS_SUN5_ENV)
+	afs_kstat_init();
+#endif
+
 	printf("found %d non-empty cache files (%d%%).\n",
 	       afs_stats_cmperf.cacheFilesReused,
 	       (100 * afs_stats_cmperf.cacheFilesReused) /
@@ -1175,41 +1452,17 @@ afs_syscall_call(long parm, long parm2, long parm3,
 	    afs_CheckServers(0, NULL);     /* check up servers */
 	}
     }
-#ifdef	AFS_SGI53_ENV
-    else if (parm == AFSOP_NFSSTATICADDR) {
-	extern int (*nfs_rfsdisptab_v2) ();
-	nfs_rfsdisptab_v2 = (int (*)())parm2;
-    } else if (parm == AFSOP_NFSSTATICADDR2) {
-	extern int (*nfs_rfsdisptab_v2) ();
-# ifdef _K64U64
-	nfs_rfsdisptab_v2 = (int (*)())((parm2 << 32) | (parm3 & 0xffffffff));
-# else /* _K64U64 */
-	nfs_rfsdisptab_v2 = (int (*)())(parm3 & 0xffffffff);
-# endif /* _K64U64 */
-    }
-# if defined(AFS_SGI62_ENV) && !defined(AFS_SGI65_ENV)
-    else if (parm == AFSOP_SBLOCKSTATICADDR2) {
-	extern int (*afs_sblockp) ();
-	extern void (*afs_sbunlockp) ();
-#  ifdef _K64U64
-	afs_sblockp = (int (*)())((parm2 << 32) | (parm3 & 0xffffffff));
-	afs_sbunlockp = (void (*)())((parm4 << 32) | (parm5 & 0xffffffff));
-#  else
-	afs_sblockp = (int (*)())(parm3 & 0xffffffff);
-	afs_sbunlockp = (void (*)())(parm5 & 0xffffffff);
-#  endif /* _K64U64 */
-    }
-# endif /* AFS_SGI62_ENV && !AFS_SGI65_ENV */
-#endif /* AFS_SGI53_ENV */
     else if (parm == AFSOP_SHUTDOWN) {
-	afs_cold_shutdown = 0;
-	if (parm2 == 1)
-	    afs_cold_shutdown = 1;
 	if (afs_globalVFS != 0) {
 	    afs_warn("AFS isn't unmounted yet! Call aborted\n");
 	    code = EACCES;
-	} else
+	} else {
+	    afs_cold_shutdown = 0;
+	    if (parm2 == 1) {
+		afs_cold_shutdown = 1;
+	    }
 	    afs_shutdown();
+	}
     } else if (parm == AFSOP_AFS_VFSMOUNT) {
 #ifdef	AFS_HPUX_ENV
 	vfsmount(parm2, parm3, parm4, parm5);
@@ -1307,6 +1560,20 @@ afs_syscall_call(long parm, long parm2, long parm3,
     } else if (parm == AFSOP_SET_RMTSYS_FLAG) {
 	afs_rmtsys_enable = parm2;
 	code = 0;
+#ifndef UKERNEL
+    } else if (parm == AFSOP_SEED_ENTROPY) {
+	unsigned char *seedbuf;
+
+	if (parm3 > 4096) {
+	    code = EFAULT;
+	} else {
+	    seedbuf = afs_osi_Alloc(parm3);
+	    AFS_COPYIN(AFSKPTR(parm2), seedbuf, parm3, code);
+	    RAND_seed(seedbuf, parm3);
+	    memset(seedbuf, 0, parm3);
+	    afs_osi_Free(seedbuf, parm3);
+	}
+#endif
     } else if (parm == AFSOP_SET_INUMCALC) {
 	switch (parm2) {
 	case AFS_INUMCALC_COMPAT:
@@ -1320,6 +1587,17 @@ afs_syscall_call(long parm, long parm2, long parm3,
 	default:
 	    code = EINVAL;
 	}
+    } else if (parm == AFSOP_SET_VOLUME_TTL) {
+	if ((parm2 < AFS_MIN_VOLUME_TTL) || (parm2 > AFS_MAX_VOLUME_TTL)) {
+	    code = EFAULT;
+	} else {
+	    afs_volume_ttl = parm2;
+	    code = 0;
+	}
+#ifdef AFS_SOCKPROXY_ENV
+    } else if (parm == AFSOP_SOCKPROXY_HANDLER) {
+	code = sockproxy_handler(AFSKPTR(parm2), AFSKPTR(parm3));
+#endif
     } else {
 	code = EINVAL;
     }
@@ -1384,6 +1662,10 @@ afs_shutdown(void)
 
     afs_shuttingdown = AFS_SHUTDOWN;
 
+#if defined(AFS_SUN5_ENV)
+    afs_kstat_shutdown();
+#endif
+
     if (afs_cold_shutdown)
 	afs_warn("afs: COLD ");
     else
@@ -1514,7 +1796,7 @@ void
 shutdown_afstest(void)
 {
     AFS_STATCNT(shutdown_afstest);
-    afs_initState = afs_termState = afs_setTime = 0;
+    afs_initState = afs_termState = 0;
     AFS_Running = afs_CB_Running = 0;
     afs_CacheInit_Done = afs_Go_Done = 0;
     if (afs_cold_shutdown) {
